﻿package  
{
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.KeyboardEvent;
	import flash.events.TimerEvent;
	import flash.geom.Point;
	import flash.ui.Keyboard;
	import flash.utils.Timer;
	
	/**
	 * ...
	 * @author ...
	 */
	public class Puyo extends Sprite
	{
		static internal const YELLOW:uint = 0xffff00;
		static internal const GREEN:uint = 0x00ff00;
		static internal const RED:uint = 0xff0000;
		static internal const BLUE:uint = 0x0000ff;
		
		static internal const RIGHT:uint = 0;
		static internal const LEFT:uint = 1;
		static internal const DOWN:uint = 2;
		static internal const UP:uint = 3;
		
		static internal const RADIUS:uint = 20;
		
		internal var color:uint;		
		internal var pair:Array;		
		
		private var fallTimer:Timer;
		
		internal var checked:int;
		
		
		private var dummy:DummyPuyo;
		
		public function Puyo(xi:uint, yi:uint, col:uint = 0x0) 
		{
			x = xi;
			y = yi;

			setOwnColor(col);
		
			if(col == 0x0) {
				addChild(dummy = new DummyPuyo(2 * RADIUS, 0, this));
				fallTimer = new Timer(300);
				fallTimer.addEventListener(TimerEvent.TIMER, fall);
				fallTimer.start();

				Main.container.stage.addEventListener(KeyboardEvent.KEY_DOWN, onKeyPress);
			}			

			checked = -1;
			pair = new Array();
			
			Main.container.addChild(this);			
		}

		
		private function onKeyPress(event:KeyboardEvent):void {

			switch(event.keyCode) {
				case Keyboard.RIGHT:
					move(RIGHT);
					if (x > PuyoStage.WIDTH || isContactAll() || (dummy.x + x) > PuyoStage.WIDTH || dummy.isContactAll())
						move(LEFT);
					break;
				case Keyboard.LEFT:
					move(LEFT);
					if (x < 0 || isContactAll() || (dummy.x + x) < 0 || dummy.isContactAll())
						move(RIGHT);
					break;					
					
				case Keyboard.DOWN:
					do {
						move(DOWN);
					} while (y < 700 && !isContactAll() && (y + dummy.y) < 700 && !dummy.isContactAll());
					move(UP);
					break;
					
				case Keyboard.UP:
					rotate();
					break;
			}
		}

		private function isOut():Boolean {
			return x < 0 || x > PuyoStage.WIDTH || y > 700;
		}		
		
		private function boundary():void {
			
			if (x < 0 || (dummy.x + x) < 0)
				x += 2 * RADIUS;
				
			else if (x > PuyoStage.WIDTH || (dummy.x + x) > PuyoStage.WIDTH)
				x -= 2 * RADIUS;
				
			else if (y > 700 || (dummy.y + y) > 700 || isContactAll() || dummy.isContactAll())
				y -= 2 * RADIUS;
			
		}
		
		private function rotate():void {

			if (dummy.x < 0) {
				move(DOWN);
				dummy.move(RIGHT);
				boundary();
			}			
			else if (dummy.x > 0) {
				move(UP);
				dummy.move(LEFT);		
				boundary();
			}			
			else if (dummy.y < 0) {
				move(LEFT);
				dummy.move(DOWN);
				boundary();
			}			
			else if (dummy.y > 0) {
				move(RIGHT);
				dummy.move(UP);
				boundary();
			}
		}

		private function setOwnColor(col:uint):void {
			
			if(col == 0x0)	
				switch(Math.floor(Math.random() * 4)) {
					case 0:
						color = YELLOW;
						break;
					case 1:
						color = GREEN;
						break;
					case 2:
						color = BLUE;
						break;
					case 3:
						color = RED;
						break;					
				}
			else
				color = col;
			
			graphics.lineStyle(1, 0xffffff);
			graphics.beginFill(color);
			graphics.drawCircle(0, 0, RADIUS);
			graphics.endFill();
		}
		
		
		internal function isContactAll():Boolean {
			for each(var v:Puyo in PuyoStage.puyoList) {
				if (v == this)
					continue;
				
				else if (isContact(v))
					return true;
			}
					
			return false;
		}
		
		private function isContact(target:Puyo):Boolean {			
			return Math.abs(y - target.y) + Math.abs(x - target.x) == 0;
		}
		
		private function fall(event:TimerEvent):void {
			move(DOWN);

			if (isContactAll() || y > 700 || dummy.isContactAll() || dummy.y + y > 700) {
				move(UP);
				Main.container.stage.removeEventListener(KeyboardEvent.KEY_DOWN, onKeyPress);
				fallTimer.reset();		

				
				newNextPuyo();
			}									
		}
		
		private function newNextPuyo():void {			

			var tmp:Puyo = new Puyo(x + dummy.x, y + dummy.y, dummy.color);
			
			PuyoStage.puyoList.push(this);
			PuyoStage.puyoList.push(tmp);
						
			do {
				tmp.move(DOWN);
			} while (tmp.y < 700 && !tmp.isContactAll());
			tmp.move(UP);
			
			do {
				move(DOWN);
			} while (y < 700 && !isContactAll());
			move(UP);	
			
			removeChild(dummy);			
			PuyoStage.reNewState();							
		}
		
				
		internal function move(direction:uint):void {
			
			switch(direction) {
				case RIGHT:
					x += 2 * RADIUS;
					break;
				case LEFT:
					x -= 2 * RADIUS;
					break;
				case DOWN:
					y += 2 * RADIUS;
					break;					
				case UP:
					y -= 2 * RADIUS;
					break;					
			}
		}
		
		internal function isPuyo(target:Puyo):Boolean {

			if ((target.color == color) && (Math.abs(y - target.y) + Math.abs(x - target.x) == 2 * RADIUS))
				return true;				
			else
				return false;
		}
		
		internal function remove():void {

			var index:uint = PuyoStage.puyoList.indexOf(this);
			PuyoStage.puyoList.splice(index, 1);
			
			fallTimer = new Timer(10);
			fallTimer.addEventListener(TimerEvent.TIMER, fadeOut);
			fallTimer.start();
		}
		
		private function fadeOut(event:TimerEvent):void {
			alpha -= 0.02;
			scaleX += 0.01;
			scaleY += 0.01;
			
			if (alpha < 0.02) {
				fallTimer.reset();
				Main.container.removeChild(this);
			}				
		}		
		
		internal function getLinkNum(parent:Puyo, chk:int):uint {
				
			checked = chk;
			var n:uint = 0;
			
			for each(var v:Puyo in pair) {
				if (parent == v || v.checked == chk)
					continue;
					
				else
					n += v.getLinkNum(this, chk);
			}
			
			return n + 1;
		}		
	}	
}
